@@ -18,6 +18,7 @@
 package org.apache.drill.exec.client;
 
 import static com.google.common.base.Preconditions.checkState;
+import static com.google.common.base.Preconditions.checkArgument;
 import static org.apache.drill.exec.proto.UserProtos.QueryResultsMode.STREAM_FULL;
 import static org.apache.drill.exec.proto.UserProtos.RunQuery.newBuilder;
 import io.netty.buffer.DrillBuf;
@@ -41,9 +42,10 @@ import org.apache.drill.common.exceptions.UserException;
 import org.apache.drill.exec.ExecConstants;
 import org.apache.drill.exec.coord.ClusterCoordinator;
 import org.apache.drill.exec.coord.zk.ZKClusterCoordinator;
+import org.apache.drill.exec.exception.OutOfMemoryException;
 import org.apache.drill.exec.memory.BufferAllocator;
-import org.apache.drill.exec.memory.OutOfMemoryException;
 import org.apache.drill.exec.memory.RootAllocatorFactory;
+import org.apache.drill.exec.proto.BitControl.PlanFragment;
 import org.apache.drill.exec.proto.CoordinationProtos.DrillbitEndpoint;
 import org.apache.drill.exec.proto.GeneralRPCProtos.Ack;
 import org.apache.drill.exec.proto.UserBitShared;
@@ -51,22 +53,41 @@ import org.apache.drill.exec.proto.UserBitShared.QueryId;
 import org.apache.drill.exec.proto.UserBitShared.QueryResult.QueryState;
 import org.apache.drill.exec.proto.UserBitShared.QueryType;
 import org.apache.drill.exec.proto.UserProtos;
+import org.apache.drill.exec.proto.UserProtos.CreatePreparedStatementReq;
+import org.apache.drill.exec.proto.UserProtos.CreatePreparedStatementResp;
+import org.apache.drill.exec.proto.UserProtos.GetCatalogsResp;
+import org.apache.drill.exec.proto.UserProtos.GetCatalogsReq;
+import org.apache.drill.exec.proto.UserProtos.GetColumnsReq;
+import org.apache.drill.exec.proto.UserProtos.GetColumnsResp;
+import org.apache.drill.exec.proto.UserProtos.GetQueryPlanFragments;
+import org.apache.drill.exec.proto.UserProtos.GetSchemasReq;
+import org.apache.drill.exec.proto.UserProtos.GetSchemasResp;
+import org.apache.drill.exec.proto.UserProtos.GetTablesReq;
+import org.apache.drill.exec.proto.UserProtos.GetTablesResp;
+import org.apache.drill.exec.proto.UserProtos.LikeFilter;
+import org.apache.drill.exec.proto.UserProtos.PreparedStatementHandle;
 import org.apache.drill.exec.proto.UserProtos.Property;
+import org.apache.drill.exec.proto.UserProtos.QueryPlanFragments;
 import org.apache.drill.exec.proto.UserProtos.RpcType;
+import org.apache.drill.exec.proto.UserProtos.RunQuery;
 import org.apache.drill.exec.proto.UserProtos.UserProperties;
 import org.apache.drill.exec.proto.helper.QueryIdHelper;
 import org.apache.drill.exec.rpc.BasicClientWithConnection.ServerConnection;
 import org.apache.drill.exec.rpc.ChannelClosedException;
+import org.apache.drill.exec.rpc.ConnectionThrottle;
 import org.apache.drill.exec.rpc.DrillRpcFuture;
 import org.apache.drill.exec.rpc.NamedThreadFactory;
 import org.apache.drill.exec.rpc.RpcConnectionHandler;
 import org.apache.drill.exec.rpc.RpcException;
 import org.apache.drill.exec.rpc.TransportCheck;
-import org.apache.drill.exec.rpc.user.ConnectionThrottle;
 import org.apache.drill.exec.rpc.user.QueryDataBatch;
 import org.apache.drill.exec.rpc.user.UserClient;
 import org.apache.drill.exec.rpc.user.UserResultsListener;
 
+import com.fasterxml.jackson.core.JsonProcessingException;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.node.ArrayNode;
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Strings;
 import com.google.common.util.concurrent.AbstractCheckedFuture;
 import com.google.common.util.concurrent.SettableFuture;
@@ -78,6 +99,7 @@ import com.google.common.util.concurrent.SettableFuture;
 public class DrillClient implements Closeable, ConnectionThrottle {
   private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(DrillClient.class);
 
+  private static final ObjectMapper objectMapper = new ObjectMapper();
   private final DrillConfig config;
   private UserClient client;
   private UserProperties props = null;
@@ -288,7 +310,7 @@ public class DrillClient implements Closeable, ConnectionThrottle {
         try {
           clusterCoordinator.close();
           clusterCoordinator = null;
-        } catch (IOException e) {
+        } catch (Exception e) {
           logger.warn("Error while closing Cluster Coordinator.", e);
         }
       }
@@ -308,19 +330,76 @@ public class DrillClient implements Closeable, ConnectionThrottle {
   }
 
   /**
-   * Submits a Logical plan for direct execution (bypasses parsing)
+   * Submits a string based query plan for execution and returns the result batches. Supported query types are:
+   * <p><ul>
+   *  <li>{@link QueryType#LOGICAL}
+   *  <li>{@link QueryType#PHYSICAL}
+   *  <li>{@link QueryType#SQL}
+   * </ul>
    *
-   * @param plan the plan to execute
+   * @param type Query type
+   * @param plan Query to execute
    * @return a handle for the query result
    * @throws RpcException
    */
   public List<QueryDataBatch> runQuery(QueryType type, String plan) throws RpcException {
+    checkArgument(type == QueryType.LOGICAL || type == QueryType.PHYSICAL || type == QueryType.SQL,
+        String.format("Only query types %s, %s and %s are supported in this API",
+            QueryType.LOGICAL, QueryType.PHYSICAL, QueryType.SQL));
     final UserProtos.RunQuery query = newBuilder().setResultsMode(STREAM_FULL).setType(type).setPlan(plan).build();
     final ListHoldingResultsListener listener = new ListHoldingResultsListener(query);
     client.submitQuery(listener, query);
     return listener.getResults();
   }
 
+  /**
+   * API to just plan a query without execution
+   * @param type
+   * @param query
+   * @param isSplitPlan - option to tell whether to return single or split plans for a query
+   * @return list of PlanFragments that can be used later on in {@link #runQuery(QueryType, List, UserResultsListener)}
+   * to run a query without additional planning
+   */
+  public DrillRpcFuture<QueryPlanFragments> planQuery(QueryType type, String query, boolean isSplitPlan) {
+    GetQueryPlanFragments runQuery = GetQueryPlanFragments.newBuilder().setQuery(query).setType(type).setSplitPlan(isSplitPlan).build();
+    return client.planQuery(runQuery);
+  }
+
+  /**
+   * Run query based on list of fragments that were supposedly produced during query planning phase. Supported
+   * query type is {@link QueryType#EXECUTION}
+   * @param type
+   * @param planFragments
+   * @param resultsListener
+   * @throws RpcException
+   */
+  public void runQuery(QueryType type, List<PlanFragment> planFragments, UserResultsListener resultsListener)
+      throws RpcException {
+    // QueryType can be only executional
+    checkArgument((QueryType.EXECUTION == type), "Only EXECUTION type query is supported with PlanFragments");
+    // setting Plan on RunQuery will be used for logging purposes and therefore can not be null
+    // since there is no Plan string provided we will create a JsonArray out of individual fragment Plans
+    ArrayNode jsonArray = objectMapper.createArrayNode();
+    for (PlanFragment fragment : planFragments) {
+      try {
+        jsonArray.add(objectMapper.readTree(fragment.getFragmentJson()));
+      } catch (IOException e) {
+        logger.error("Exception while trying to read PlanFragment JSON for %s", fragment.getHandle().getQueryId(), e);
+        throw new RpcException(e);
+      }
+    }
+    final String fragmentsToJsonString;
+    try {
+      fragmentsToJsonString = objectMapper.writeValueAsString(jsonArray);
+    } catch (JsonProcessingException e) {
+      logger.error("Exception while trying to get JSONString from Array of individual Fragments Json for %s", e);
+      throw new RpcException(e);
+    }
+    final UserProtos.RunQuery query = newBuilder().setType(type).addAllFragments(planFragments)
+        .setPlan(fragmentsToJsonString)
+        .setResultsMode(STREAM_FULL).build();
+    client.submitQuery(resultsListener, query);
+  }
 
   /*
    * Helper method to generate the UserCredentials message from the properties.
@@ -355,6 +434,154 @@ public class DrillClient implements Closeable, ConnectionThrottle {
     return client.send(RpcType.RESUME_PAUSED_QUERY, queryId, Ack.class);
   }
 
+  /**
+   * Get the list of catalogs in <code>INFORMATION_SCHEMA.CATALOGS</code> table satisfying the given filters.
+   *
+   * @param catalogNameFilter Filter on <code>catalog name</code>. Pass null to apply no filter.
+   * @return
+   */
+  public DrillRpcFuture<GetCatalogsResp> getCatalogs(LikeFilter catalogNameFilter) {
+    final GetCatalogsReq.Builder reqBuilder = GetCatalogsReq.newBuilder();
+    if (catalogNameFilter != null) {
+      reqBuilder.setCatalogNameFilter(catalogNameFilter);
+    }
+
+    return client.send(RpcType.GET_CATALOGS, reqBuilder.build(), GetCatalogsResp.class);
+  }
+
+  /**
+   * Get the list of schemas in <code>INFORMATION_SCHEMA.SCHEMATA</code> table satisfying the given filters.
+   *
+   * @param catalogNameFilter Filter on <code>catalog name</code>. Pass null to apply no filter.
+   * @param schemaNameFilter Filter on <code>schema name</code>. Pass null to apply no filter.
+   * @return
+   */
+  public DrillRpcFuture<GetSchemasResp> getSchemas(LikeFilter catalogNameFilter, LikeFilter schemaNameFilter) {
+    final GetSchemasReq.Builder reqBuilder = GetSchemasReq.newBuilder();
+    if (catalogNameFilter != null) {
+      reqBuilder.setCatalogNameFilter(catalogNameFilter);
+    }
+
+    if (schemaNameFilter != null) {
+      reqBuilder.setSchameNameFilter(schemaNameFilter);
+    }
+
+    return client.send(RpcType.GET_SCHEMAS, reqBuilder.build(), GetSchemasResp.class);
+  }
+
+  /**
+   * Get the list of tables in <code>INFORMATION_SCHEMA.TABLES</code> table satisfying the given filters.
+   *
+   * @param catalogNameFilter Filter on <code>catalog name</code>. Pass null to apply no filter.
+   * @param schemaNameFilter Filter on <code>schema name</code>. Pass null to apply no filter.
+   * @param tableNameFilter Filter in <code>table name</code>. Pass null to apply no filter.
+   * @return
+   */
+  public DrillRpcFuture<GetTablesResp> getTables(LikeFilter catalogNameFilter, LikeFilter schemaNameFilter,
+      LikeFilter tableNameFilter) {
+    final GetTablesReq.Builder reqBuilder = GetTablesReq.newBuilder();
+    if (catalogNameFilter != null) {
+      reqBuilder.setCatalogNameFilter(catalogNameFilter);
+    }
+
+    if (schemaNameFilter != null) {
+      reqBuilder.setSchameNameFilter(schemaNameFilter);
+    }
+
+    if (tableNameFilter != null) {
+      reqBuilder.setTableNameFilter(tableNameFilter);
+    }
+
+    return client.send(RpcType.GET_TABLES, reqBuilder.build(), GetTablesResp.class);
+  }
+
+  /**
+   * Get the list of columns in <code>INFORMATION_SCHEMA.COLUMNS</code> table satisfying the given filters.
+   *
+   * @param catalogNameFilter Filter on <code>catalog name</code>. Pass null to apply no filter.
+   * @param schemaNameFilter Filter on <code>schema name</code>. Pass null to apply no filter.
+   * @param tableNameFilter Filter in <code>table name</code>. Pass null to apply no filter.
+   * @param columnNameFilter Filter in <code>column name</code>. Pass null to apply no filter.
+   * @return
+   */
+  public DrillRpcFuture<GetColumnsResp> getColumns(LikeFilter catalogNameFilter, LikeFilter schemaNameFilter,
+      LikeFilter tableNameFilter, LikeFilter columnNameFilter) {
+    final GetColumnsReq.Builder reqBuilder = GetColumnsReq.newBuilder();
+    if (catalogNameFilter != null) {
+      reqBuilder.setCatalogNameFilter(catalogNameFilter);
+    }
+
+    if (schemaNameFilter != null) {
+      reqBuilder.setSchameNameFilter(schemaNameFilter);
+    }
+
+    if (tableNameFilter != null) {
+      reqBuilder.setTableNameFilter(tableNameFilter);
+    }
+
+    if (columnNameFilter != null) {
+      reqBuilder.setColumnNameFilter(columnNameFilter);
+    }
+
+    return client.send(RpcType.GET_COLUMNS, reqBuilder.build(), GetColumnsResp.class);
+  }
+
+  /**
+   * Create a prepared statement for given <code>query</code>.
+   *
+   * @param query
+   * @return
+   */
+  public DrillRpcFuture<CreatePreparedStatementResp> createPreparedStatement(final String query) {
+    final CreatePreparedStatementReq req =
+        CreatePreparedStatementReq.newBuilder()
+            .setSqlQuery(query)
+            .build();
+
+    return client.send(RpcType.CREATE_PREPARED_STATEMENT, req, CreatePreparedStatementResp.class);
+  }
+
+  /**
+   * Execute the given prepared statement.
+   *
+   * @param preparedStatementHandle Prepared statement handle returned in response to
+   *                                {@link #createPreparedStatement(String)}.
+   * @param resultsListener {@link UserResultsListener} instance for listening for query results.
+   */
+  public void executePreparedStatement(final PreparedStatementHandle preparedStatementHandle,
+      final UserResultsListener resultsListener) {
+    final RunQuery runQuery = newBuilder()
+        .setResultsMode(STREAM_FULL)
+        .setType(QueryType.PREPARED_STATEMENT)
+        .setPreparedStatementHandle(preparedStatementHandle)
+        .build();
+    client.submitQuery(resultsListener, runQuery);
+  }
+
+  /**
+   * Execute the given prepared statement and return the results.
+   *
+   * @param preparedStatementHandle Prepared statement handle returned in response to
+   *                                {@link #createPreparedStatement(String)}.
+   * @return List of {@link QueryDataBatch}s. It is responsibility of the caller to release query data batches.
+   * @throws RpcException
+   */
+  @VisibleForTesting
+  public List<QueryDataBatch> executePreparedStatement(final PreparedStatementHandle preparedStatementHandle)
+      throws RpcException {
+    final RunQuery runQuery = newBuilder()
+        .setResultsMode(STREAM_FULL)
+        .setType(QueryType.PREPARED_STATEMENT)
+        .setPreparedStatementHandle(preparedStatementHandle)
+        .build();
+
+    final ListHoldingResultsListener resultsListener = new ListHoldingResultsListener(runQuery);
+
+    client.submitQuery(resultsListener, runQuery);
+
+    return resultsListener.getResults();
+  }
+
   /**
    * Submits a Logical plan for direct execution (bypasses parsing)
    *
