@@ -22,11 +22,15 @@ import org.apache.drill.exec.expr.fn.FunctionImplementationRegistry;
 import org.apache.drill.exec.ops.QueryContext;
 import org.apache.drill.exec.server.options.OptionManager;
 import org.apache.drill.exec.server.options.OptionValidator;
+import org.apache.drill.exec.server.options.TypeValidators;
 import org.apache.drill.exec.server.options.TypeValidators.BooleanValidator;
 import org.apache.drill.exec.server.options.TypeValidators.LongValidator;
+import org.apache.drill.exec.server.options.TypeValidators.DoubleValidator;
 import org.apache.drill.exec.server.options.TypeValidators.PositiveLongValidator;
 import org.apache.drill.exec.server.options.TypeValidators.RangeDoubleValidator;
 import org.apache.drill.exec.server.options.TypeValidators.RangeLongValidator;
+import org.apache.drill.exec.server.options.TypeValidators.MinRangeDoubleValidator;
+import org.apache.drill.exec.server.options.TypeValidators.MaxRangeDoubleValidator;
 import org.apache.calcite.plan.Context;
 
 public class PlannerSettings implements Context{
@@ -72,14 +76,29 @@ public class PlannerSettings implements Context{
   public static final OptionValidator HASH_JOIN_SWAP_MARGIN_FACTOR = new RangeDoubleValidator("planner.join.hash_join_swap_margin_factor", 0, 100, 10d);
   public static final String ENABLE_DECIMAL_DATA_TYPE_KEY = "planner.enable_decimal_data_type";
   public static final OptionValidator ENABLE_DECIMAL_DATA_TYPE = new BooleanValidator(ENABLE_DECIMAL_DATA_TYPE_KEY, false);
-  public static final OptionValidator HEP_JOIN_OPT = new BooleanValidator("planner.enable_hep_join_opt", true);
+  public static final OptionValidator HEP_OPT = new BooleanValidator("planner.enable_hep_opt", true);
+  public static final OptionValidator HEP_PARTITION_PRUNING = new BooleanValidator("planner.enable_hep_partition_pruning", true);
   public static final OptionValidator PLANNER_MEMORY_LIMIT = new RangeLongValidator("planner.memory_limit",
       INITIAL_OFF_HEAP_ALLOCATION_IN_BYTES, MAX_OFF_HEAP_ALLOCATION_IN_BYTES, DEFAULT_MAX_OFF_HEAP_ALLOCATION_IN_BYTES);
+  public static final String UNIONALL_DISTRIBUTE_KEY = "planner.enable_unionall_distribute";
+  public static final BooleanValidator UNIONALL_DISTRIBUTE = new BooleanValidator(UNIONALL_DISTRIBUTE_KEY, false);
 
   public static final OptionValidator IDENTIFIER_MAX_LENGTH =
       new RangeLongValidator("planner.identifier_max_length", 128 /* A minimum length is needed because option names are identifiers themselves */,
                               Integer.MAX_VALUE, DEFAULT_IDENTIFIER_MAX_LENGTH);
 
+  public static final DoubleValidator FILTER_MIN_SELECTIVITY_ESTIMATE_FACTOR =
+          new MinRangeDoubleValidator("planner.filter.min_selectivity_estimate_factor",
+          0.0, 1.0, 0.0d, "planner.filter.max_selectivity_estimate_factor");
+  public static final DoubleValidator FILTER_MAX_SELECTIVITY_ESTIMATE_FACTOR =
+          new MaxRangeDoubleValidator("planner.filter.max_selectivity_estimate_factor",
+          0.0, 1.0, 1.0d, "planner.filter.min_selectivity_estimate_factor");
+
+  public static final String TYPE_INFERENCE_KEY = "planner.enable_type_inference";
+  public static final BooleanValidator TYPE_INFERENCE = new BooleanValidator(TYPE_INFERENCE_KEY, true);
+  public static final LongValidator IN_SUBQUERY_THRESHOLD =
+      new PositiveLongValidator("planner.in_subquery_threshold", Integer.MAX_VALUE, 20); /* Same as Calcite's default IN List subquery size */
+
   public OptionManager options = null;
   public FunctionImplementationRegistry functionImplementationRegistry = null;
 
@@ -172,7 +191,9 @@ public class PlannerSettings implements Context{
     return options.getOption(HASH_JOIN_SWAP.getOptionName()).bool_val;
   }
 
-  public boolean isHepJoinOptEnabled() { return options.getOption(HEP_JOIN_OPT.getOptionName()).bool_val;}
+  public boolean isHepPartitionPruningEnabled() { return options.getOption(HEP_PARTITION_PRUNING.getOptionName()).bool_val;}
+
+  public boolean isHepOptEnabled() { return options.getOption(HEP_OPT.getOptionName()).bool_val;}
 
   public double getHashJoinSwapMarginFactor() {
     return options.getOption(HASH_JOIN_SWAP_MARGIN_FACTOR.getOptionName()).float_val / 100d;
@@ -206,6 +227,26 @@ public class PlannerSettings implements Context{
     return INITIAL_OFF_HEAP_ALLOCATION_IN_BYTES;
   }
 
+  public double getFilterMinSelectivityEstimateFactor() {
+    return options.getOption(FILTER_MIN_SELECTIVITY_ESTIMATE_FACTOR);
+  }
+
+  public double getFilterMaxSelectivityEstimateFactor(){
+    return options.getOption(FILTER_MAX_SELECTIVITY_ESTIMATE_FACTOR);
+  }
+
+  public boolean isTypeInferenceEnabled() {
+    return options.getOption(TYPE_INFERENCE);
+  }
+
+  public long getInSubqueryThreshold() {
+    return options.getOption(IN_SUBQUERY_THRESHOLD);
+  }
+
+  public boolean isUnionAllDistributeEnabled() {
+    return options.getOption(UNIONALL_DISTRIBUTE);
+  }
+
   @Override
   public <T> T unwrap(Class<T> clazz) {
     if(clazz == PlannerSettings.class){
