@@ -23,7 +23,6 @@ import java.util.Collection;
 import java.util.List;
 import java.util.concurrent.TimeUnit;
 
-import org.apache.calcite.plan.RelOptCluster;
 import org.apache.calcite.plan.RelOptPlanner;
 import org.apache.calcite.plan.RelOptRule;
 import org.apache.calcite.plan.RelOptUtil;
@@ -209,16 +208,20 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
   protected DrillRel convertToDrel(final RelNode relNode) throws SqlUnsupportedException, RelConversionException {
     // Check if this is a "limit 0 " query.
     final boolean limitZero = FindLimit0Visitor.containsLimit0(relNode);
+
     if (context.getOptions().getOption(ExecConstants.EARLY_LIMIT0_OPT) &&
         context.getPlannerSettings().isTypeInferenceEnabled() &&
         limitZero) {
-      // disable distributed mode
-      context.getPlannerSettings().forceSingleMode();
       // if the schema is known, return the schema directly
       final DrillRel shorterPlan;
       if ((shorterPlan = FindLimit0Visitor.getDirectScanRelIfFullySchemaed(relNode)) != null) {
         return shorterPlan;
       }
+
+      if (FindHardDistributionScans.canForceSingleMode(relNode)) {
+        // disable distributed mode
+        context.getPlannerSettings().forceSingleMode();
+      }
     }
 
     try {
@@ -269,7 +272,8 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
       } else {
 
         // If the query contains a limit 0 clause, disable distributed mode since it is overkill for determining schema.
-        if (FindLimit0Visitor.containsLimit0(convertedRelNodeWithSum0)) {
+        if (FindLimit0Visitor.containsLimit0(convertedRelNodeWithSum0) &&
+            FindHardDistributionScans.canForceSingleMode(convertedRelNodeWithSum0)) {
           context.getPlannerSettings().forceSingleMode();
           if (context.getOptions().getOption(ExecConstants.LATE_LIMIT0_OPT)) {
             return FindLimit0Visitor.addLimitOnTopOfLeafNodes(drillRel);
@@ -396,17 +400,13 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
       final RelMetadataProvider cachingMetaDataProvider = new CachingRelMetadataProvider(
           ChainedRelMetadataProvider.of(list), planner);
 
-      // Save original metadataProvider before call HepPlanner call.
-      RelMetadataProvider oldProvider = input.getCluster().getMetadataProvider();
-      input.getCluster().setMetadataProvider(cachingMetaDataProvider);
+      // Modify RelMetaProvider for every RelNode in the SQL operator Rel tree.
+      input.accept(new MetaDataProviderModifier(cachingMetaDataProvider));
       planner.setRoot(input);
       if (!input.getTraitSet().equals(targetTraits)) {
         planner.changeTraits(input, toTraits);
       }
       output = planner.findBestExp();
-
-      // Restore to original metadataProvider after HepPlanner call.
-      input.getCluster().setMetadataProvider(oldProvider);
       break;
     }
     case VOLCANO:
@@ -671,7 +671,10 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
       projections.add(b.makeInputRef(rel, i));
     }
 
-    final List<String> fieldNames2 = SqlValidatorUtil.uniquify(validatedRowType.getFieldNames(), SqlValidatorUtil.F_SUGGESTER2);
+    final List<String> fieldNames2 = SqlValidatorUtil.uniquify(
+        validatedRowType.getFieldNames(),
+        SqlValidatorUtil.F_SUGGESTER2,
+        rel.getCluster().getTypeFactory().getTypeSystem().isSchemaCaseSensitive());
 
     RelDataType newRowType = RexUtil.createStructType(rel.getCluster().getTypeFactory(), projections, fieldNames2);
 
