@@ -17,29 +17,26 @@
  */
 package org.apache.drill.exec.store.parquet.columnreaders;
 
-import static org.apache.parquet.format.converter.ParquetMetadataConverter.fromParquetStatistics;
 import static org.apache.parquet.column.Encoding.valueOf;
-import io.netty.buffer.ByteBuf;
-import io.netty.buffer.DrillBuf;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.concurrent.TimeUnit;
 
 import org.apache.drill.common.exceptions.ExecutionSetupException;
 import org.apache.drill.exec.store.parquet.ColumnDataReader;
 import org.apache.drill.exec.store.parquet.ParquetFormatPlugin;
+import org.apache.drill.exec.store.parquet.ParquetReaderStats;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-
 import org.apache.parquet.bytes.BytesInput;
 import org.apache.parquet.column.Dictionary;
 import org.apache.parquet.column.Encoding;
 import org.apache.parquet.column.ValuesType;
 import org.apache.parquet.column.page.DictionaryPage;
-import org.apache.parquet.column.statistics.Statistics;
 import org.apache.parquet.column.values.ValuesReader;
 import org.apache.parquet.column.values.dictionary.DictionaryValuesReader;
 import org.apache.parquet.format.PageHeader;
@@ -52,6 +49,10 @@ import org.apache.parquet.hadoop.metadata.CompressionCodecName;
 import org.apache.parquet.schema.PrimitiveType;
 
 import com.google.common.base.Preconditions;
+import com.google.common.base.Stopwatch;
+
+import io.netty.buffer.ByteBuf;
+import io.netty.buffer.DrillBuf;
 
 // class to keep track of the read position of variable length columns
 final class PageReader {
@@ -59,7 +60,7 @@ final class PageReader {
 
   public static final ParquetMetadataConverter METADATA_CONVERTER = ParquetFormatPlugin.parquetMetadataConverter;
 
-  private final ColumnReader parentColumnReader;
+  private final ColumnReader<?> parentColumnReader;
   private final ColumnDataReader dataReader;
 
   // buffer to store bytes of current page
@@ -98,22 +99,26 @@ final class PageReader {
 
   int currentPageCount = -1;
 
+  private FSDataInputStream inputStream;
+
   // These need to be held throughout reading of the entire column chunk
   List<ByteBuf> allocatedDictionaryBuffers;
 
   private final CodecFactory codecFactory;
 
+  private final ParquetReaderStats stats;
+
   PageReader(ColumnReader<?> parentStatus, FileSystem fs, Path path, ColumnChunkMetaData columnChunkMetaData)
     throws ExecutionSetupException{
     this.parentColumnReader = parentStatus;
     allocatedDictionaryBuffers = new ArrayList<ByteBuf>();
     codecFactory = parentColumnReader.parentReader.getCodecFactory();
-
+    this.stats = parentColumnReader.parentReader.parquetReaderStats;
     long start = columnChunkMetaData.getFirstDataPageOffset();
     try {
-      FSDataInputStream f = fs.open(path);
-      this.dataReader = new ColumnDataReader(f, start, columnChunkMetaData.getTotalSize());
-      loadDictionaryIfExists(parentStatus, columnChunkMetaData, f);
+      inputStream  = fs.open(path);
+      this.dataReader = new ColumnDataReader(inputStream, start, columnChunkMetaData.getTotalSize());
+      loadDictionaryIfExists(parentStatus, columnChunkMetaData, inputStream);
 
     } catch (IOException e) {
       throw new ExecutionSetupException("Error opening or reading metadata for parquet file at location: "
@@ -124,9 +129,15 @@ final class PageReader {
 
   private void loadDictionaryIfExists(final ColumnReader<?> parentStatus,
       final ColumnChunkMetaData columnChunkMetaData, final FSDataInputStream f) throws IOException {
+    Stopwatch timer = Stopwatch.createUnstarted();
     if (columnChunkMetaData.getDictionaryPageOffset() > 0) {
       f.seek(columnChunkMetaData.getDictionaryPageOffset());
+      long start=f.getPos();
+      timer.start();
       final PageHeader pageHeader = Util.readPageHeader(f);
+      long timeToRead = timer.elapsed(TimeUnit.MICROSECONDS);
+      long pageHeaderBytes=f.getPos()-start;
+      this.updateStats(pageHeader, "Page Header", start, timeToRead, pageHeaderBytes, pageHeaderBytes);
       assert pageHeader.type == PageType.DICTIONARY_PAGE;
       readDictionaryPage(pageHeader, parentStatus);
     }
@@ -138,7 +149,7 @@ final class PageReader {
     int uncompressedSize = pageHeader.getUncompressed_page_size();
 
     final DrillBuf dictionaryData = allocateDictionaryBuffer(uncompressedSize);
-    readPage(compressedSize, uncompressedSize, dictionaryData);
+    readPage(pageHeader, compressedSize, uncompressedSize, dictionaryData);
 
     DictionaryPage page = new DictionaryPage(
         asBytesInput(dictionaryData, 0, uncompressedSize),
@@ -149,20 +160,30 @@ final class PageReader {
     this.dictionary = page.getEncoding().initDictionary(parentStatus.columnDescriptor, page);
   }
 
-  public void readPage(int compressedSize, int uncompressedSize, DrillBuf dest) throws IOException {
+  public void readPage(PageHeader pageHeader, int compressedSize, int uncompressedSize, DrillBuf dest) throws IOException {
+    Stopwatch timer = Stopwatch.createUnstarted();
+    long timeToRead;
+    long start=inputStream.getPos();
     if (parentColumnReader.columnChunkMetaData.getCodec() == CompressionCodecName.UNCOMPRESSED) {
+      timer.start();
       dataReader.loadPage(dest, compressedSize);
+      timeToRead = timer.elapsed(TimeUnit.MICROSECONDS);
+      this.updateStats(pageHeader, "Page Read", start, timeToRead, compressedSize, uncompressedSize);
     } else {
       final DrillBuf compressedData = allocateTemporaryBuffer(compressedSize);
       try {
-        dataReader.loadPage(compressedData, compressedSize);
-        codecFactory.getDecompressor(parentColumnReader.columnChunkMetaData
-            .getCodec()).decompress(
-            compressedData.nioBuffer(0, compressedSize),
-            compressedSize,
-            dest.nioBuffer(0, uncompressedSize),
-            uncompressedSize);
-
+      timer.start();
+      dataReader.loadPage(compressedData, compressedSize);
+      timeToRead = timer.elapsed(TimeUnit.MICROSECONDS);
+      timer.reset();
+      this.updateStats(pageHeader, "Page Read", start, timeToRead, compressedSize, compressedSize);
+      start = inputStream.getPos();
+      timer.start();
+      codecFactory.getDecompressor(parentColumnReader.columnChunkMetaData
+          .getCodec()).decompress(compressedData.nioBuffer(0, compressedSize), compressedSize,
+          dest.nioBuffer(0, uncompressedSize), uncompressedSize);
+        timeToRead = timer.elapsed(TimeUnit.MICROSECONDS);
+        this.updateStats(pageHeader, "Decompress", start, timeToRead, compressedSize, uncompressedSize);
       } finally {
         compressedData.release();
       }
@@ -180,7 +201,7 @@ final class PageReader {
    * @throws java.io.IOException
    */
   public boolean next() throws IOException {
-
+    Stopwatch timer = Stopwatch.createUnstarted();
     currentPageCount = -1;
     valuesRead = 0;
     valuesReadyToRead = 0;
@@ -196,7 +217,15 @@ final class PageReader {
     // TODO - figure out if we need multiple dictionary pages, I believe it may be limited to one
     // I think we are clobbering parts of the dictionary if there can be multiple pages of dictionary
     do {
+      long start=inputStream.getPos();
+      timer.start();
       pageHeader = dataReader.readPageHeader();
+      long timeToRead = timer.elapsed(TimeUnit.MICROSECONDS);
+      this.updateStats(pageHeader, "Page Header Read", start, timeToRead, 0,0);
+      logger.trace("ParquetTrace,{},{},{},{},{},{},{},{}","Page Header Read","",
+          this.parentColumnReader.parentReader.hadoopPath,
+          this.parentColumnReader.columnDescriptor.toString(), start, 0, 0, timeToRead);
+      timer.reset();
       if (pageHeader.getType() == PageType.DICTIONARY_PAGE) {
         readDictionaryPage(pageHeader, parentColumnReader);
       }
@@ -207,13 +236,10 @@ final class PageReader {
     allocatePageData(pageHeader.getUncompressed_page_size());
     int compressedSize = pageHeader.getCompressed_page_size();
     int uncompressedSize = pageHeader.getUncompressed_page_size();
-    readPage(compressedSize, uncompressedSize, pageData);
+    readPage(pageHeader, compressedSize, uncompressedSize, pageData);
 
     currentPageCount = pageHeader.data_page_header.num_values;
 
-    final int uncompressedPageSize = pageHeader.uncompressed_page_size;
-    final Statistics<?> stats = fromParquetStatistics(pageHeader.data_page_header.getStatistics(), parentColumnReader
-        .getColumnDescriptor().getType());
     final Encoding rlEncoding = METADATA_CONVERTER.getEncoding(pageHeader.data_page_header.repetition_level_encoding);
 
     final Encoding dlEncoding = METADATA_CONVERTER.getEncoding(pageHeader.data_page_header.definition_level_encoding);
@@ -298,6 +324,37 @@ final class PageReader {
     return currentPageCount != -1;
   }
 
+  private void updateStats(PageHeader pageHeader, String op, long start, long time, long bytesin, long bytesout) {
+    String pageType = "Data Page";
+    if (pageHeader.type == PageType.DICTIONARY_PAGE) {
+      pageType = "Dictionary Page";
+    }
+    logger.trace("ParquetTrace,{},{},{},{},{},{},{},{}", op, pageType.toString(),
+        this.parentColumnReader.parentReader.hadoopPath,
+        this.parentColumnReader.columnDescriptor.toString(), start, bytesin, bytesout, time);
+    if (pageHeader.type != PageType.DICTIONARY_PAGE) {
+      if (bytesin == bytesout) {
+        this.stats.timePageLoads += time;
+        this.stats.numPageLoads++;
+        this.stats.totalPageReadBytes += bytesin;
+      } else {
+        this.stats.timePagesDecompressed += time;
+        this.stats.numPagesDecompressed++;
+        this.stats.totalDecompressedBytes += bytesin;
+      }
+    } else {
+      if (bytesin == bytesout) {
+        this.stats.timeDictPageLoads += time;
+        this.stats.numDictPageLoads++;
+        this.stats.totalDictPageReadBytes += bytesin;
+      } else {
+        this.stats.timeDictPagesDecompressed += time;
+        this.stats.numDictPagesDecompressed++;
+        this.stats.totalDictDecompressedBytes += bytesin;
+      }
+    }
+  }
+
   public void clearBuffers() {
     if (pageData != null) {
       pageData.release();
